#include <Arduino.h>
#include <TinyGPS.h>
#include <ESP8266WiFi.h>
#include <ESPAsyncTCP.h>
#include <FS.h>
#include <ESPAsyncWebServer.h>

const int sound = 0; // выход на пищалку
const int p_sound = 2; // выход PWM на управление питанием пищалки

int r_speed; // приведенное в целое значение текущей скорости движения
int V1; // значение скорости предупреждения
int V2; // значение скорости превышения 
int S1 = 100; //длительность начального сигнала предупреждения о превышении 0,1 сек
int S2 = 1000; //длительность сигнала о превышении 1 сек
int S; // значение длительности звучания сигнала
int Sp; // значение длительности паузы в звучании сигнала
int Volsound; // уровень громкости сигнала в процентах
int volume; // длительность ШИМ уровня громкости сигнала

const char* ssid     = "GPS-speed";
const char* password = "";

AsyncWebServer server(80);

String inputMessage = "0.0";
String lastSpeed;

const char* PARAM_INT1 = "inputInt1";
const char* PARAM_INT2 = "inputInt2";
const char* PARAM_INT3 = "inputInt3";
const char index_html[] PROGMEM = R"rawliteral(

<!DOCTYPE HTML><html><head>
  <title>ESP Input Form</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    h2 {font-size: 3.0rem;}
    p {font-size: 3.0rem;}
    body {max-width: 600px; margin:0px auto; padding-bottom: 25px;}
    .switch {position: relative; display: inline-block; width: 120px; height: 68px} 
    .switch input {display: none}
    .slider {position: absolute; top: 0; left: 0; right: 0; bottom: 0; background-color: #ccc; border-radius: 6px}
    .slider:before {position: absolute; content: ""; height: 52px; width: 52px; left: 8px; bottom: 8px; background-color: #fff; -webkit-transition: .4s; transition: .4s; border-radius: 3px}
    input:checked+.slider {background-color: #b30000}
    input:checked+.slider:before {-webkit-transform: translateX(52px); -ms-transform: translateX(52px); transform: translateX(52px)}
  </style>
  <h1>%SPEED% км/ч</h1>
   
  <script>
    function submitMessage() {
      alert("Saved value to ESP SPIFFS");
      setTimeout(function(){ document.location.reload(false); }, 500);   
    }
  </script></head><body>
  <form action="/get" target="hidden-form">
    Предупреждение, км/ч (установлено <font style="color: Red"><font size="5">%inputInt1%</font></font>): <input type="number" name="inputInt1">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <form action="/get" target="hidden-form">
    Превышение, км/ч &ensp; &ensp; &thinsp; (установлено <font style="color: Red"><font size="5">%inputInt2%</font></font>): <input type="number " name="inputInt2">
    <input type="submit" value="Submit" onclick="submitMessage()">
    </form><br>
  <form action="/get" target="hidden-form">
    Громкость звука, &percnt; &ensp; &ensp; (установлено <font style="color: Red"><font size="5">%inputInt3%</font></font>): <input type="number" name="inputInt3">
    <input type="submit" value="Submit" onclick="submitMessage()">
  </form><br>
  <iframe style="display:none" name="hidden-form"></iframe>
</body></html>)rawliteral";

void notFound(AsyncWebServerRequest *request) {
  request->send(404, "text/plain", "Not found");
}

String readFile(fs::FS &fs, const char * path){
  //Serial.printf("Reading file: %s\r\n", path);
  File file = fs.open(path, "r");
  if(!file || file.isDirectory()){
    //Serial.println("- empty file or failed to open file");
    return String();
  }
  //Serial.println("- read from file:");
  String fileContent;
  while(file.available()){
    fileContent+=String((char)file.read());
  }
  file.close();
  //Serial.println(fileContent);
  return fileContent;
}

void writeFile(fs::FS &fs, const char * path, const char * message){
  //Serial.printf("Writing file: %s\r\n", path);
  File file = fs.open(path, "w");
  if(!file){
    //Serial.println("- failed to open file for writing");
    return;
  }
  if(file.print(message)){
    //Serial.println("- file written");
  } else {
    //Serial.println("- write failed");
  }
  file.close();
}

// Replaces placeholder with stored values
String processor(const String& var){
  if(var == "inputInt1"){
    return readFile(SPIFFS, "/inputInt1.txt");
  }
  if(var == "inputInt2"){
    return readFile(SPIFFS, "/inputInt2.txt");
  }
  if(var == "inputInt3"){
    return readFile(SPIFFS, "/inputInt3.txt");
  }
  if(var == "SPEED"){
  return lastSpeed;
  }
  return String();
}

TinyGPS gps;
static void smartdelay(unsigned long ms);

void setup()
{
  Serial.begin(9600);
  
  pinMode (sound, OUTPUT); // пищалка
  digitalWrite(sound, HIGH); // пищалка выключена
  pinMode (p_sound, OUTPUT); // питание пищалки
  analogWriteFreq(10000); // частота ШИМ 10 кГц
  analogWrite(p_sound, 0); // питание пищалки выключено
   
  // Initialize SPIFFS
  if(!SPIFFS.begin()){
  Serial.println("An Error has occurred while mounting SPIFFS");
  return;
  }

  //Serial.print("Setting AP (Access Point)…");
  // Remove the password parameter, if you want the AP (Access Point) to be open
  WiFi.softAP(ssid, password);

  IPAddress IP = WiFi.softAPIP();
  //Serial.print("AP IP address: ");
  //Serial.println(IP);

  // Print ESP8266 Local IP Address
  //Serial.println(WiFi.localIP());

  // Send web page with input fields to client
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });

  // Send a GET request to <ESP_IP>/get?inputString=<inputMessage>
  server.on("/get", HTTP_GET, [] (AsyncWebServerRequest *request) {
    String inputMessage;
    // GET inputInt1 value on <ESP_IP>/get?inputInt1=<inputMessage>
      if (request->hasParam(PARAM_INT1)) {
      inputMessage = request->getParam(PARAM_INT1)->value();
      writeFile(SPIFFS, "/inputInt1.txt", inputMessage.c_str());
    }
    // GET inputInt2 value on <ESP_IP>/get?inputInt2=<inputMessage>
    else if (request->hasParam(PARAM_INT2)) {
      inputMessage = request->getParam(PARAM_INT2)->value();
      writeFile(SPIFFS, "/inputInt2.txt", inputMessage.c_str());
    }
    // GET inputInt3 value on <ESP_IP>/get?inputInt3=<inputMessage>
    else if (request->hasParam(PARAM_INT3)) {
      inputMessage = request->getParam(PARAM_INT3)->value();
      writeFile(SPIFFS, "/inputInt3.txt", inputMessage.c_str());
    }
    else {
      inputMessage = "No message sent";
    }
    //Serial.println(inputMessage);
    request->send(200, "text/text", inputMessage);
  });
  server.onNotFound(notFound);
  server.begin();
}

void loop()
{
  // To access your stored values on inputInt1, inputInt2, inputInt3
  V1 = readFile(SPIFFS, "/inputInt1.txt").toInt(); // читаем значение скорости предупреждения
  V2 = readFile(SPIFFS, "/inputInt2.txt").toInt(); // читаем значение скорости превышения 
  Volsound = readFile(SPIFFS, "/inputInt3.txt").toInt(); // читаем значение уровня громкости сигнала в процентах

  volume = Volsound * 128 / 100 + 127; // вычисляем громкость пищалки, от 50% до 100%
  analogWrite(p_sound, volume); // устанавливаем напряжение питания пищалки 
        
  float speed = gps.f_speed_kmph();
  //Serial.println(speed);
  lastSpeed = String(speed);

  // Расчитываем частоты звуковых посылок по формуле
  // SV = (((S2-S1)/(V2-V1)) * r_speed) - (((S2-S1)/(V2-V1))*V1 - S1)
  // где:
  // S1 - длительность сигнала предупреждения о превышении скорости
  // S2 - длительность сигнала предупреждения о превышении скорости
  // V1 - значение скорости предупреждения
  // V2 - значение скорости превышения 
  // r_speed - приведенное в целое значение текущей скорости движения
   
  r_speed = (int)speed;
  //r_speed = 15; 
  
  if (r_speed >= V1) {
    S = (((S2 - S1)/(V2-V1)) * r_speed) - ((((S2-S1)/(V2-V1)) * V1) - S1); // расчитанное значение длительности сигнала 
    Serial.println(S);
    if (S > S2) S = S2; // длительность сигнала не может больше, чем S2
    if (S < S1) S = S1; // длительность сигнала не может меньше, чем S1   
    digitalWrite(sound, LOW);
    smartdelay(S); // длительность сигнала
    digitalWrite(sound, HIGH); 
    Sp = 1000 - S; // пауза сигнала
    smartdelay(Sp);  
  }
  else digitalWrite(sound, HIGH); // отключение сигнала
  smartdelay(1000);

}  
  
static void smartdelay(unsigned long ms) {
  unsigned long start = millis();
  do {
    while (Serial.available())
      gps.encode(Serial.read());
  } while (millis() - start < ms);
}
